@@ -146,7 +146,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                  gtk.keysyms.Control_R : MASK_CTRL}
 
     # Identifiers for MsgArea messages
-    (MSG_SAME, MSG_SLOW_HIGHLIGHT) = list(range(2))
+    (MSG_SAME, MSG_SLOW_HIGHLIGHT, MSG_SYNCPOINTS) = list(range(3))
 
     __gsignals__ = {
         'next-conflict-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (bool, bool)),
@@ -216,6 +216,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.animating_chunks = [[] for buf in self.textbuffer]
         for buf in self.textbuffer:
             buf.create_tag("inline")
+            buf.connect("notify::has-selection",
+                        self.update_text_actions_sensitivity)
 
         actions = (
             ("MakePatch", None, _("Format as Patch..."), None,
@@ -352,7 +354,9 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.fill_colors = {"insert"  : lookup("insert-bg", "DarkSeaGreen1"),
                             "delete"  : lookup("insert-bg", "DarkSeaGreen1"),
                             "conflict": lookup("conflict-bg", "Pink"),
-                            "replace" : lookup("replace-bg", "#ddeeff")}
+                            "replace" : lookup("replace-bg", "#ddeeff"),
+                            "current-chunk-highlight":
+                                lookup("current-chunk-highlight", '#ffffff')}
         self.line_colors = {"insert"  : lookup("insert-outline", "#77f077"),
                             "delete"  : lookup("insert-outline", "#77f077"),
                             "conflict": lookup("conflict-outline", "#f0768b"),
@@ -712,6 +716,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.on_cursor_position_changed(view.get_buffer(), None, True)
         self._set_save_action_sensitivity()
         self._set_merge_action_sensitivity()
+        self.update_text_actions_sensitivity()
 
     def on_textview_focus_out_event(self, view, event):
         self._set_merge_action_sensitivity()
@@ -719,8 +724,9 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
     def _after_text_modified(self, buffer, startline, sizechange):
         if self.num_panes > 1:
             pane = self.textbuffer.index(buffer)
-            self.linediffer.change_sequence(pane, startline, sizechange,
-                                            self.buffer_filtered)
+            if not self.syncpoints:
+                self.linediffer.change_sequence(pane, startline, sizechange,
+                                                self.buffer_filtered)
             # FIXME: diff-changed signal for the current buffer would be cleaner
             focused_pane = self._get_focused_pane()
             if focused_pane != -1:
@@ -920,6 +926,20 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 line = cursor_it.get_line() + 1
                 self._open_files([self.textbuffer[pane].data.filename], line)
 
+    def update_text_actions_sensitivity(self, *args):
+        widget = self.focus_pane
+        if not widget:
+            cut, copy, paste = False, False, False
+        else:
+            cut = copy = widget.get_buffer().get_has_selection()
+            # Ideally, this would check whether the clipboard included
+            # something pasteable. However, there is no changed signal.
+            # widget.get_clipboard(
+            #    gtk.gdk.SELECTION_CLIPBOARD).wait_is_text_available()
+            paste = widget.get_editable()
+        for action, sens in zip(("Cut", "Copy", "Paste"), (cut, copy, paste)):
+            self.main_actiongroup.get_action(action).set_sensitive(sens)
+
     def get_selected_text(self):
         """Returns selected text of active pane"""
         pane = self._get_focused_pane()
@@ -1433,7 +1453,9 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 context.set_source_color(self.fill_colors[change[0]])
                 context.fill_preserve()
                 if self.linediffer.locate_chunk(pane, change[1])[0] == self.cursor.chunk:
-                    context.set_source_rgba(1.0, 1.0, 1.0, 0.5)
+                    h = self.fill_colors['current-chunk-highlight']
+                    context.set_source_rgba(
+                        h.red_float, h.green_float, h.blue_float, 0.5)
                     context.fill_preserve()
 
             context.set_source_color(self.line_colors[change[0]])
@@ -1450,8 +1472,11 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             context.restore()
 
         for syncpoint in [p[pane] for p in self.syncpoints]:
-            if bounds[0] <= syncpoint <= bounds[1]:
-                ypos = textview.get_y_for_line_num(syncpoint) - visible.y
+            if not syncpoint:
+                continue
+            syncline = textbuffer.get_iter_at_mark(syncpoint).get_line()
+            if bounds[0] <= syncline <= bounds[1]:
+                ypos = textview.get_y_for_line_num(syncline) - visible.y
                 context.rectangle(-0.5, ypos - 0.5, width + 1, 1)
                 context.set_source_color(self.syncpoint_color)
                 context.stroke()
@@ -1915,19 +1940,48 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             syncpoint = [None] * self.num_panes
         cursor_it = self.textbuffer[pane].get_iter_at_mark(
             self.textbuffer[pane].get_insert())
-        syncpoint[pane] = cursor_it.get_line()
+        syncpoint[pane] = self.textbuffer[pane].create_mark(None, cursor_it)
         self.syncpoints.append(syncpoint)
 
+        def make_line_retriever(pane, marks):
+            buf = self.textbuffer[pane]
+            mark = marks[pane]
+
+            def get_line_for_mark():
+                return buf.get_iter_at_mark(mark).get_line()
+            return get_line_for_mark
+
         valid_points = [p for p in self.syncpoints if all(p)]
         if valid_points and self.num_panes == 2:
             self.linediffer.syncpoints = [
-                ((p[1], p[0]), ) for p in valid_points]
+                ((make_line_retriever(1, p), make_line_retriever(0, p)), )
+                for p in valid_points
+            ]
         elif valid_points and self.num_panes == 3:
             self.linediffer.syncpoints = [
-                ((p[1], p[0]), (p[1], p[2])) for p in valid_points]
+                ((make_line_retriever(1, p), make_line_retriever(0, p)),
+                 (make_line_retriever(1, p), make_line_retriever(2, p)))
+                for p in valid_points
+            ]
+
+        if valid_points:
+            for mgr in self.msgarea_mgr:
+                msgarea = mgr.new_from_text_and_icon(
+                    gtk.STOCK_DIALOG_INFO,
+                    _("Live comparison updating disabled"),
+                    _("Live updating of comparisons is disabled when "
+                      "synchronization points are active. You can still "
+                      "manually refresh the comparison, and live updates will "
+                      "resume when synchronization points are cleared."))
+                mgr.set_msg_id(FileDiff.MSG_SYNCPOINTS)
+                msgarea.show_all()
+
         self.refresh_comparison()
 
     def clear_sync_points(self, action):
         self.syncpoints = []
         self.linediffer.syncpoints = []
+        for mgr in self.msgarea_mgr:
+            if mgr.get_msg_id() == FileDiff.MSG_SYNCPOINTS:
+                mgr.clear()
         self.refresh_comparison()
