@@ -143,7 +143,8 @@ class PreferencesDialog(gnomeglade.Component):
 
     def __init__(self, parent, prefs):
         gnomeglade.Component.__init__(self, paths.ui_dir("preferences.ui"),
-                                      "preferencesdialog", ["adjustment1"])
+                                      "preferencesdialog",
+                                      ["adjustment1", "adjustment2", "fileorderstore"])
         self.widget.set_transient_for(parent)
         self.prefs = prefs
         if not self.prefs.use_custom_font:
@@ -162,7 +163,7 @@ class PreferencesDialog(gnomeglade.Component):
             self.checkbutton_use_syntax_highlighting.set_active( self.prefs.use_syntax_highlighting )
         else:
             no_sourceview_text = \
-                _("Only available if you have gnome-python-desktop installed")
+                _("Only available if you have PyGtkSourceView 2 installed")
             for w in (self.checkbutton_spaces_instead_of_tabs,
                       self.checkbutton_show_line_numbers,
                       self.checkbutton_use_syntax_highlighting,
@@ -220,6 +221,27 @@ class PreferencesDialog(gnomeglade.Component):
         self.combo_timestamp.set_active(active_idx)
         self.combo_timestamp.lock = False
 
+        self.combo_file_order.set_active(
+            1 if self.prefs.vc_left_is_local else 0)
+
+        if srcviewer.gsv is not None:
+            self.checkbutton_show_commit_margin.set_active(
+                self.prefs.vc_show_commit_margin)
+            self.spinbutton_commit_margin.set_value(
+                self.prefs.vc_commit_margin)
+            self.checkbutton_break_commit_lines.set_sensitive(
+                self.prefs.vc_show_commit_margin)
+            self.checkbutton_break_commit_lines.set_active(
+                self.prefs.vc_break_commit_message)
+        else:
+            no_sourceview_text = \
+                _("Only available if you have PyGtkSourceView 2 installed")
+            for w in (self.checkbutton_show_commit_margin,
+                      self.spinbutton_commit_margin,
+                      self.checkbutton_break_commit_lines):
+                w.set_sensitive(False)
+                w.set_tooltip_text(no_sourceview_text)
+
         self.widget.show()
 
     def on_fontpicker_font_set(self, picker):
@@ -265,6 +287,20 @@ class PreferencesDialog(gnomeglade.Component):
         # Called on "activate" and "focus-out-event"
         self.prefs.edit_command_custom = entry.props.text
 
+    def on_checkbutton_show_line_numbers_toggled(self, check):
+        self.prefs.show_line_numbers = check.get_active()
+
+    def on_checkbutton_show_commit_margin_toggled(self, check):
+        show_margin = check.get_active()
+        self.prefs.vc_show_commit_margin = show_margin
+        self.checkbutton_break_commit_lines.set_sensitive(show_margin)
+
+    def on_spinbutton_commit_margin_value_changed(self, spin):
+        self.prefs.vc_commit_margin = int(spin.get_value())
+
+    def on_checkbutton_break_commit_lines_toggled(self, check):
+        self.prefs.vc_break_commit_message = check.get_active()
+
     #
     # filters
     #
@@ -285,6 +321,10 @@ class PreferencesDialog(gnomeglade.Component):
             resolution = combo.get_model()[combo.get_active_iter()][1]
             self.prefs.dirdiff_time_resolution_ns = resolution
 
+    def on_combo_file_order_changed(self, combo):
+        file_order = combo.get_model()[combo.get_active_iter()][0]
+        self.prefs.vc_left_is_local = True if file_order else False
+
     def on_response(self, dialog, response_id):
         self.widget.destroy()
 
@@ -344,6 +384,12 @@ class MeldPreferences(prefs.Preferences):
                                           "permissions 0"]),
         "dirdiff_shallow_comparison" : prefs.Value(prefs.BOOL, False),
         "dirdiff_time_resolution_ns" : prefs.Value(prefs.INT, 100),
+
+        "vc_show_commit_margin": prefs.Value(prefs.BOOL, True),
+        "vc_commit_margin": prefs.Value(prefs.INT, 72),
+        "vc_break_commit_message": prefs.Value(prefs.BOOL, False),
+
+        "vc_left_is_local": prefs.Value(prefs.BOOL, False),
     }
 
     def __init__(self):
