@@ -92,7 +92,10 @@ class _gtksourceview2(_srcviewer):
 
     def get_language_from_file(self, filename):
         f = gio.File(filename)
-        info = f.query_info(gio.FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE)
+        try:
+            info = f.query_info(gio.FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE)
+        except gio.Error:
+            return None
         content_type = info.get_content_type()
         return self.get_language_manager().guess_language(filename,
                                                           content_type)
@@ -169,6 +172,8 @@ class nullsourceview(_srcviewer):
             set_show_line_numbers = lambda *args: None
             set_insert_spaces_instead_of_tabs = lambda *args: None
             set_draw_spaces = lambda *args: None
+            set_right_margin_position = lambda *args: None
+            set_show_right_margin = lambda *args: None
         gobject.type_register(NullTextView)
 
         self.GtkTextView = NullTextView
