@@ -42,8 +42,6 @@ class Vc(_vc.CachedVc):
     CMD = "git"
     NAME = "Git"
     VC_DIR = ".git"
-    PATCH_STRIP_NUM = 1
-    PATCH_INDEX_RE = "^diff --git [ac]/(.*) [bw]/.*$"
     GIT_DIFF_FILES_RE = ":(\d+) (\d+) [a-z0-9]+ [a-z0-9]+ ([ADMU])\t(.*)"
 
     VC_COLUMNS = (_vc.DATA_NAME, _vc.DATA_STATE, _vc.DATA_OPTIONS)
@@ -80,21 +78,9 @@ class Vc(_vc.CachedVc):
     def commit_command(self, message):
         return [self.CMD, "commit", "-m", message]
 
-    def diff_command(self):
-        return [self.CMD, "diff", "--relative", "HEAD"]
-
-    def update_command(self):
-        return [self.CMD, "pull"]
-
     def add_command(self):
         return [self.CMD, "add"]
 
-    def remove_command(self, force=0):
-        return [self.CMD, "rm"]
-
-    def revert_command(self):
-        return [self.CMD, "checkout"]
-
     # Prototyping VC interface version 2
 
     def update_actions_for_paths(self, path_states, actions):
@@ -129,9 +115,16 @@ class Vc(_vc.CachedVc):
         unpushed_commits = sum(len(v) for v in branch_refs.values())
         if unpushed_commits:
             if unpushed_branches > 1:
-                label = _("%d unpushed commits in %d branches") % \
-                    (unpushed_commits, unpushed_branches)
+                # Translators: First %s is replaced by translated "%d unpushed
+                # commits", second %s is replaced by translated "%d branches"
+                label = _("%s in %s") % (
+                    ngettext("%d unpushed commit", "%d unpushed commits",
+                             unpushed_commits) % unpushed_commits,
+                    ngettext("%d branch", "%d branches",
+                             unpushed_branches) % unpushed_branches)
             else:
+                # Translators: These messages cover the case where there is
+                # only one branch, and are not part of another message.
                 label = ngettext("%d unpushed commit", "%d unpushed commits",
                                  unpushed_commits) % (unpushed_commits)
         else:
@@ -188,6 +181,10 @@ class Vc(_vc.CachedVc):
         command = [self.CMD, 'push']
         runner(command, [], refresh=True, working_dir=self.root)
 
+    def remove(self, runner, files):
+        command = [self.CMD, 'rm', '-r']
+        runner(command, files, refresh=True, working_dir=self.root)
+
     def revert(self, runner, files):
         exists = [f for f in files if os.path.exists(f)]
         missing = [f for f in files if not os.path.exists(f)]
