@@ -1,20 +1,20 @@
-### Copyright (C) 2002-2006 Stephen Kennedy <stevek@gnome.org>
-### Copyright (C) 2010-2012 Kai Willadsen <kai.willadsen@gmail.com>
+# Copyright (C) 2002-2006 Stephen Kennedy <stevek@gnome.org>
+# Copyright (C) 2010-2013 Kai Willadsen <kai.willadsen@gmail.com>
 
-### This program is free software; you can redistribute it and/or modify
-### it under the terms of the GNU General Public License as published by
-### the Free Software Foundation; either version 2 of the License, or
-### (at your option) any later version.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
-### This program is distributed in the hope that it will be useful,
-### but WITHOUT ANY WARRANTY; without even the implied warranty of
-### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-### GNU General Public License for more details.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 
-### You should have received a copy of the GNU General Public License
-### along with this program; if not, write to the Free Software
-### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
-### USA.
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+# USA.
 
 from __future__ import print_function
 
@@ -37,6 +37,7 @@ from . import tree
 from . import vc
 from .ui import emblemcellrenderer
 from .ui import gnomeglade
+from .ui import vcdialogs
 from meld.vc import _null
 
 
@@ -78,85 +79,6 @@ _temp_dirs, _temp_files = [], []
 atexit.register(cleanup_temp)
 
 
-class CommitDialog(gnomeglade.Component):
-
-    def __init__(self, parent):
-        gnomeglade.Component.__init__(self, paths.ui_dir("vcview.ui"),
-                                      "commitdialog")
-        self.parent = parent
-        self.widget.set_transient_for(parent.widget.get_toplevel())
-        selected = parent._get_selected_files()
-
-        try:
-            to_commit = parent.vc.get_files_to_commit(selected)
-            topdir = parent.vc.root
-            if to_commit:
-                to_commit = ["\t" + s for s in to_commit]
-            else:
-                to_commit = ["\t" + _("No files will be committed")]
-        except NotImplementedError:
-            topdir = _commonprefix(selected)
-            to_commit = ["\t" + s[len(topdir) + 1:] for s in selected]
-        self.changedfiles.set_text("(in %s)\n%s" %
-                                   (topdir, "\n".join(to_commit)))
-
-        fontdesc = pango.FontDescription(self.parent.prefs.get_current_font())
-        self.textview.modify_font(fontdesc)
-        commit_prefill = self.parent.vc.get_commit_message_prefill()
-        if commit_prefill:
-            buf = self.textview.get_buffer()
-            buf.set_text(commit_prefill)
-            buf.place_cursor(buf.get_start_iter())
-
-        # Try and make the textview wide enough for a standard 80-character
-        # commit message.
-        context = self.textview.get_pango_context()
-        metrics = context.get_metrics(fontdesc, context.get_language())
-        char_width = metrics.get_approximate_char_width()
-        self.textview.set_size_request(80 * pango.PIXELS(char_width), -1)
-
-        self.widget.show_all()
-
-    def run(self):
-        self.previousentry.set_active(-1)
-        self.textview.grab_focus()
-        response = self.widget.run()
-        if response == gtk.RESPONSE_OK:
-            buf = self.textview.get_buffer()
-            msg = buf.get_text(*buf.get_bounds(), include_hidden_chars=False)
-            self.parent._command_on_selected(
-                self.parent.vc.commit_command(msg))
-            if msg.strip():
-                self.previousentry.prepend_history(msg)
-        self.widget.destroy()
-
-    def on_previousentry_activate(self, gentry):
-        idx = gentry.get_active()
-        if idx != -1:
-            model = gentry.get_model()
-            buf = self.textview.get_buffer()
-            buf.set_text(model[idx][1])
-
-
-class PushDialog(gnomeglade.Component):
-
-    def __init__(self, parent):
-        gnomeglade.Component.__init__(self, paths.ui_dir("vcview.ui"),
-                                      "pushdialog")
-        self.parent = parent
-        self.widget.set_transient_for(parent.widget.get_toplevel())
-        self.widget.show_all()
-
-    def run(self):
-        # TODO: Ask the VC for a more informative label for what will happen.
-        # In git, this is probably the parsed output of push --dry-run.
-
-        response = self.widget.run()
-        if response == gtk.RESPONSE_OK:
-            self.parent.vc.push(self.parent._command)
-        self.widget.destroy()
-
-
 class ConsoleStream(object):
 
     def __init__(self, textview):
@@ -212,7 +134,6 @@ entry_ignored  = lambda x: (x.state == tree.STATE_IGNORED) or x.isdir
 class VcView(melddoc.MeldDoc, gnomeglade.Component):
     # Map action names to VC commands and required arguments list
     action_vc_cmds_map = {
-        "VcCompare": ("diff_command", ()),
         "VcCommit": ("commit_command", ("",)),
         "VcUpdate": ("update_command", ()),
         "VcPush": ("push", (lambda *args, **kwargs: None, )),
@@ -303,7 +224,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         selection.set_mode(gtk.SELECTION_MULTIPLE)
         selection.connect("changed", self.on_treeview_selection_changed)
         self.treeview.set_headers_visible(1)
-        self.treeview.set_search_equal_func(self.treeview_search_cb)
+        self.treeview.set_search_equal_func(self.model.treeview_search_cb)
         self.current_path, self.prev_path, self.next_path = None, None, None
 
         self.column_name_map = {}
@@ -380,8 +301,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             col.set_visible(data_id in self.vc.VC_COLUMNS)
 
     def update_actions_sensitivity(self):
-        """Disable actions that use not implemented VC plugin methods
-        """
+        """Disable actions that use not implemented VC plugin methods"""
         valid_vc_actions = ["VcDeleteLocally"]
         for action_name, (meth_name, args) in self.action_vc_cmds_map.items():
             action = self.actiongroup.get_action(action_name)
@@ -408,6 +328,8 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             err_str = ""
 
             def vc_installed(cmd):
+                if not cmd:
+                    return True
                 try:
                     return not vc._vc.call(["which", cmd])
                 except OSError:
@@ -432,14 +354,15 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 self.combobox_vcs.get_model().append(
                     [_("%s (%s)") % (avc.NAME, err_str), avc, False])
             else:
-                self.combobox_vcs.get_model().append([avc.NAME, avc, True])
+                name = avc.NAME or _("None")
+                self.combobox_vcs.get_model().append([name, avc, True])
 
         if not valid_vcs:
             # If we didn't get any valid vcs then fallback to null
             null_vcs = _null.Vc(vcs[0].location)
             vcs.append(null_vcs)
             self.combobox_vcs.get_model().insert(
-                0, [null_vcs.NAME, null_vcs, True])
+                0, [_("None"), null_vcs, True])
             default_active = 0
 
         if default_active == -1:
@@ -449,7 +372,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 default_active = 0
 
         # If we only have the null VC, give a better error message.
-        if (len(vcs) == 1 and vcs[0].CMD == "true") or (len(valid_vcs) == 0):
+        if (len(vcs) == 1 and not vcs[0].CMD) or (len(valid_vcs) == 0):
             tooltip = _("No valid version control system found in this folder")
         elif len(vcs) == 1:
             tooltip = _("Only one version control system found in this folder")
@@ -477,7 +400,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         self.model.clear()
         self.fileentry.set_filename(location)
         self.fileentry.prepend_history(location)
-        it = self.model.add_entries( None, [location] )
+        it = self.model.add_entries(None, [location])
         self.treeview.grab_focus()
         self.treeview.get_selection().select_iter(it)
         self.model.set_path_state(it, 0, tree.STATE_NORMAL, isdir=1)
@@ -493,7 +416,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 col = self.model.column_index(COL_OPTIONS, 0)
                 self.model.set_value(root, col,
                                      self.vc.get_commits_to_push_summary())
-            except AttributeError:
+            except NotImplementedError:
                 pass
 
             self.scheduler.add_task(self._search_recursively_iter(root))
@@ -510,51 +433,44 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         self.label_changed()
 
     def _search_recursively_iter(self, iterstart):
-        yield _("[%s] Scanning %s") % (self.label_text,"")
-        rootpath = self.model.get_path( iterstart  )
-        rootname = self.model.value_path( self.model.get_iter(rootpath), 0 )
-        prefixlen = 1 + len( self.model.value_path( self.model.get_iter_root(), 0 ) )
-        todo = [ (rootpath, rootname) ]
+        rootname = self.model.value_path(iterstart, 0)
+        prefixlen = len(self.location) + 1
+        todo = [(self.model.get_path(iterstart), rootname)]
 
+        flattened = self.actiongroup.get_action("VcFlatten").get_active()
         active_action = lambda a: self.actiongroup.get_action(a).get_active()
-        filters = [a[1] for a in self.state_actions.values() if \
+        filters = [a[1] for a in self.state_actions.values() if
                    active_action(a[0]) and a[1]]
 
-        def showable(entry):
-            for f in filters:
-                if f(entry): return 1
-        recursive = self.actiongroup.get_action("VcFlatten").get_active()
+        yield _("Scanning %s") % rootname
         self.vc.cache_inventory(rootname)
-        while len(todo):
-            todo.sort() # depth first
-            path, name = todo.pop(0)
-            if path:
-                it = self.model.get_iter( path )
-                root = self.model.value_path( it, 0 )
-            else:
-                it = self.model.get_iter_root()
-                root = name
-            yield _("[%s] Scanning %s") % (self.label_text, root[prefixlen:])
+        while todo:
+            # This needs to happen sorted and depth-first in order for our row
+            # references to remain valid while we traverse.
+            todo.sort()
+            treepath, path = todo.pop(0)
+            it = self.model.get_iter(treepath)
+            yield _("Scanning %s") % path[prefixlen:]
 
-            entries = [f for f in self.vc.listdir(root) if showable(f)]
-            differences = 0
+            entries = self.vc.listdir(path)
+            entries = [e for e in entries if any(f(e) for f in filters)]
             for e in entries:
-                differences |= (e.state != tree.STATE_NORMAL)
-                if e.isdir and recursive:
-                    todo.append( (None, e.path) )
+                if e.isdir and flattened:
+                    todo.append(((0,), e.path))
                     continue
+
                 child = self.model.add_entries(it, [e.path])
-                self._update_item_state( child, e, root[prefixlen:] )
+                self._update_item_state(child, e, path[prefixlen:])
                 if e.isdir:
-                    todo.append( (self.model.get_path(child), None) )
-            if not recursive: # expand parents
-                if len(entries) == 0:
+                    todo.append((self.model.get_path(child), e.path))
+
+            if flattened:
+                self.treeview.expand_row((0,), 0)
+            else:
+                if not entries:
                     self.model.add_empty(it, _("(Empty)"))
-                if differences or len(path)==1:
-                    self.treeview.expand_to_path(path)
-            else: # just the root
-                self.treeview.expand_row( (0,), 0)
-        self.vc.uncache_inventory()
+                if any(e.state != tree.STATE_NORMAL for e in entries):
+                    self.treeview.expand_to_path(treepath)
 
     def on_fileentry_activate(self, fileentry):
         path = fileentry.get_full_path()
@@ -570,76 +486,47 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             if self.treeview.row_expanded(path):
                 self.treeview.collapse_row(path)
             else:
-                self.treeview.expand_row(path,0)
+                self.treeview.expand_row(path, 0)
         else:
             path = self.model.value_path(it, 0)
-            self.run_diff( [path] )
+            self.run_diff(path)
 
-    def run_diff_iter(self, path_list):
-        silent_error = hasattr(self.vc, 'switch_to_external_diff')
-        retry_diff = True
-        while retry_diff:
-            retry_diff = False
+    def run_diff(self, path):
+        if os.path.isdir(path):
+            self.emit("create-diff", [path], {})
+            return
 
-            yield _("[%s] Fetching differences") % self.label_text
-            diffiter = self._command_iter(self.vc.diff_command(), path_list, 0)
-            diff = None
-            while type(diff) != type(()):
-                diff = next(diffiter)
-                yield 1
-            prefix, patch = diff[0], diff[1]
+        left_is_local = self.prefs.vc_left_is_local
 
-            yield _("[%s] Applying patch") % self.label_text
-            if patch:
-                applied = self.show_patch(prefix, patch, silent=silent_error)
-                if not applied and silent_error:
-                    silent_error = False
-                    self.vc.switch_to_external_diff()
-                    retry_diff = True
+        if self.vc.get_entry(path).state == tree.STATE_CONFLICT and \
+                hasattr(self.vc, 'get_path_for_conflict'):
+            # We create new temp files for other, base and this, and
+            # then set the output to the current file.
+            if left_is_local:
+                conflicts = (tree.CONFLICT_THIS, tree.CONFLICT_MERGED,
+                             tree.CONFLICT_OTHER)
             else:
-                for path in path_list:
-                    self.emit("create-diff", [path], {})
+                conflicts = (tree.CONFLICT_OTHER, tree.CONFLICT_MERGED,
+                             tree.CONFLICT_THIS)
+            diffs = [self.vc.get_path_for_conflict(path, conflict=c)
+                     for c in conflicts]
+            temps = [p for p, is_temp in diffs if is_temp]
+            diffs = [p for p, is_temp in diffs]
+            kwargs = {
+                'auto_merge': False,
+                'merge_output': path,
+            }
+        else:
+            comp_path = self.vc.get_path_for_repo_file(path)
+            temps = [comp_path]
+            diffs = [path, comp_path] if left_is_local else [comp_path, path]
+            kwargs = {}
 
-    def run_diff(self, path_list):
-        try:
-            for path in path_list:
-                if os.path.isdir(path):
-                    self.emit("create-diff", [path], {})
-                    continue
+        for temp_file in temps:
+            os.chmod(temp_file, 0o444)
+            _temp_files.append(temp_file)
 
-                kwargs = {}
-                vc_file = self.vc.lookup_files(
-                    [], [(os.path.basename(path), path)])[1][0]
-                if vc_file.state == tree.STATE_CONFLICT:
-                    # We use auto merge, so we create a new temp file
-                    # for other, base and this, then set the output to
-                    # the current file.
-
-                    conflicts = (tree.CONFLICT_OTHER, tree.CONFLICT_MERGED,
-                                 tree.CONFLICT_THIS)
-                    diffs = [self.vc.get_path_for_conflict(path, conflict=c)
-                             for c in conflicts]
-                    for conflict_path, is_temp in diffs:
-                        # If this is the actual file, don't touch it.
-                        if conflict_path != path and is_temp:
-                            os.chmod(conflict_path, 0o444)
-                            _temp_files.append(conflict_path)
-                    # create-diff expects only the paths
-                    diffs = [p for p, is_temp in diffs]
-
-                    # If we want to use auto-merge or use the merged
-                    # output given by the VCS
-                    kwargs['auto_merge'] = False
-                    kwargs['merge_output'] = path
-                else:
-                    comp_path = self.vc.get_path_for_repo_file(path)
-                    os.chmod(comp_path, 0o444)
-                    _temp_files.append(comp_path)
-                    diffs = [comp_path, path]
-                self.emit("create-diff", diffs, kwargs)
-        except NotImplementedError:
-            for path in path_list:
-                self.scheduler.add_task(self.run_diff_iter([path]), atfront=1)
+        self.emit("create-diff", diffs, kwargs)
 
     def on_treeview_popup_menu(self, treeview):
         time = gtk.get_current_event_time()
@@ -670,7 +557,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
 
     def on_filter_state_toggled(self, button):
         active_action = lambda a: self.actiongroup.get_action(a).get_active()
-        active_filters = [a for a in self.state_actions if \
+        active_filters = [a for a in self.state_actions if
                           active_action(self.state_actions[a][0])]
 
         if set(active_filters) == set(self.state_filters):
@@ -733,8 +620,8 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         elif len(files) == 1 and os.path.isdir(files[0]):
             workdir = self.vc.get_working_directory(files[0])
         else:
-            workdir = self.vc.get_working_directory( _commonprefix(files) )
-        files = [ relpath(workdir, f) for f in files ]
+            workdir = self.vc.get_working_directory(_commonprefix(files))
+        files = [relpath(workdir, f) for f in files]
         r = None
         self.consolestream.command(misc.shelljoin(command + files) + " (in %s)\n" % workdir)
         readiter = misc.read_pipe_iter(command + files, self.consolestream,
@@ -770,10 +657,10 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             self._command_on_selected(self.vc.update_command())
 
     def on_button_push_clicked(self, obj):
-        PushDialog(self).run()
+        vcdialogs.PushDialog(self).run()
 
     def on_button_commit_clicked(self, obj):
-        CommitDialog(self).run()
+        vcdialogs.CommitDialog(self).run()
 
     def on_button_add_clicked(self, obj):
         # This is an evil hack to let CVS and SVN < 1.7 deal with the
@@ -791,10 +678,35 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             self._command_on_selected(self.vc.add_command())
 
     def on_button_remove_clicked(self, obj):
-        self._command_on_selected(self.vc.remove_command())
+        selected = self._get_selected_files()
+        if any(os.path.isdir(p) for p in selected):
+            # TODO: Improve and reuse this dialog for the non-VC delete action
+            dialog = gtk.MessageDialog(
+                parent=self.widget.get_toplevel(),
+                flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+                type=gtk.MESSAGE_WARNING,
+                message_format=_("Remove folder and all its files?"))
+            dialog.format_secondary_text(
+                _("This will remove all selected files and folders, and all "
+                  "files within any selected folders, from version control."))
+
+            dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
+            dialog.add_button(_("_Remove"), gtk.RESPONSE_OK)
+            response = dialog.run()
+            dialog.destroy()
+            if response != gtk.RESPONSE_OK:
+                return
+
+        try:
+            self.vc.remove(self._command, self._get_selected_files())
+        except NotImplementedError:
+            self._command_on_selected(self.vc.remove_command())
 
     def on_button_resolved_clicked(self, obj):
-        self._command_on_selected(self.vc.resolved_command())
+        try:
+            self.vc.resolve(self._command, self._get_selected_files())
+        except NotImplementedError:
+            self._command_on_selected(self.vc.resolved_command())
 
     def on_button_revert_clicked(self, obj):
         try:
@@ -814,94 +726,29 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                             buttonstype=gtk.BUTTONS_OK_CANCEL) == gtk.RESPONSE_OK:
                         shutil.rmtree(name)
             except OSError as e:
-                misc.run_dialog(_("Error removing %s\n\n%s.") % (name,e), parent = self)
+                misc.run_dialog(_("Error removing %s\n\n%s.") % (name, e),
+                                parent=self)
         workdir = _commonprefix(files)
         self.refresh_partial(workdir)
 
     def on_button_diff_clicked(self, obj):
         files = self._get_selected_files()
-        if len(files):
-            self.run_diff(files)
+        for f in files:
+            self.run_diff(f)
 
     def open_external(self):
         self._open_files(self._get_selected_files())
 
-    def show_patch(self, prefix, patch, silent=False):
-        if vc._vc.call(["which", "patch"]):
-            primary = _("Patch tool not found")
-            secondary = _("Meld needs the <i>patch</i> tool to be installed "
-                          "to perform comparisons in %s repositories. Please "
-                          "install <i>patch</i> and try again.") % self.vc.NAME
-
-            msgarea = self.msgarea_mgr.new_from_text_and_icon(
-                        gtk.STOCK_DIALOG_ERROR, primary, secondary)
-            msgarea.add_button(_("Hi_de"), gtk.RESPONSE_CLOSE)
-            msgarea.connect("response", lambda *args: self.msgarea_mgr.clear())
-            msgarea.show_all()
-            return False
-
-        tmpdir = tempfile.mkdtemp("-meld")
-        _temp_dirs.append(tmpdir)
-
-        diffs = []
-        for fname in self.vc.get_patch_files(patch):
-            destfile = os.path.join(tmpdir,fname)
-            destdir = os.path.dirname( destfile )
-
-            if not os.path.exists(destdir):
-                os.makedirs(destdir)
-            pathtofile = os.path.join(prefix, fname)
-            try:
-                shutil.copyfile( pathtofile, destfile)
-            except IOError: # it is missing, create empty file
-                open(destfile,"w").close()
-            diffs.append( (destfile, pathtofile) )
-
-        patchcmd = self.vc.patch_command(tmpdir)
-        try:
-            result = misc.write_pipe(patchcmd, patch, error=misc.NULL)
-        except OSError:
-            result = 1
-
-        if result == 0:
-            for d in diffs:
-                os.chmod(d[0], 0o444)
-                self.emit("create-diff", d, {})
-            return True
-        elif not silent:
-            primary = _("Error fetching original comparison file")
-            secondary = _("Meld couldn't obtain the original version of your "
-                          "comparison file. If you are using the most recent "
-                          "version of Meld, please report a bug, including as "
-                          "many details as possible.")
-
-            msgarea = self.msgarea_mgr.new_from_text_and_icon(
-                        gtk.STOCK_DIALOG_ERROR, primary, secondary)
-            msgarea.add_button(_("Hi_de"), gtk.RESPONSE_CLOSE)
-            msgarea.add_button(_("Report a bug"), gtk.RESPONSE_OK)
-
-            def patch_error_cb(msgarea, response):
-                if response == gtk.RESPONSE_OK:
-                    bug_url = "https://bugzilla.gnome.org/enter_bug.cgi?" + \
-                              "product=meld"
-                    misc.open_uri(bug_url)
-                else:
-                    self.msgarea_mgr.clear()
-
-            msgarea.connect("response", patch_error_cb)
-            msgarea.show_all()
-
-        return False
-
     def refresh(self):
-        self.set_location( self.model.value_path( self.model.get_iter_root(), 0 ) )
+        self.set_location(self.model.value_path(self.model.get_iter_root(), 0))
 
     def refresh_partial(self, where):
         if not self.actiongroup.get_action("VcFlatten").get_active():
-            it = self.find_iter_by_name( where )
+            it = self.find_iter_by_name(where)
             if it:
-                newiter = self.model.insert_after( None, it)
-                self.model.set_value(newiter, self.model.column_index( tree.COL_PATH, 0), where)
+                newiter = self.model.insert_after(None, it)
+                self.model.set_value(
+                    newiter, self.model.column_index(tree.COL_PATH, 0), where)
                 self.model.set_path_state(newiter, 0, tree.STATE_NORMAL, True)
                 self.model.remove(it)
                 self.treeview.grab_focus()
@@ -909,12 +756,14 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                 self.scheduler.add_task(self._search_recursively_iter(newiter))
                 self.scheduler.add_task(self.on_treeview_selection_changed)
                 self.scheduler.add_task(self.on_treeview_cursor_changed)
-        else: # XXX fixme
+        else:
+            # XXX fixme
             self.refresh()
 
     def _update_item_state(self, it, vcentry, location):
         e = vcentry
         self.model.set_path_state(it, 0, e.state, e.isdir)
+
         def setcol(col, val):
             self.model.set_value(it, self.model.column_index(col, 0), val)
         setcol(COL_LOCATION, location)
@@ -965,17 +814,14 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             self.console_hbox.show()
             self.console_show_box.hide()
 
-    def on_consoleview_populate_popup(self, text, menu):
-        item = gtk.ImageMenuItem(gtk.STOCK_CLEAR)
-        def activate(*args):
-            buf = text.get_buffer()
-            buf.delete( buf.get_start_iter(), buf.get_end_iter() )
-        item.connect("activate", activate)
-        item.show()
-        menu.insert( item, 0 )
-        item = gtk.SeparatorMenuItem()
-        item.show()
-        menu.insert( item, 1 )
+    def on_consoleview_populate_popup(self, textview, menu):
+        buf = textview.get_buffer()
+        clear_cb = lambda *args: buf.delete(*buf.get_bounds())
+        clear_action = gtk.ImageMenuItem(gtk.STOCK_CLEAR)
+        clear_action.connect("activate", clear_cb)
+        menu.insert(clear_action, 0)
+        menu.insert(gtk.SeparatorMenuItem(), 1)
+        menu.show_all()
 
     def on_treeview_cursor_changed(self, *args):
         cursor_path, cursor_col = self.treeview.get_cursor()
@@ -1030,23 +876,3 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
 
     def on_find_activate(self, *extra):
         self.treeview.emit("start-interactive-search")
-
-    def treeview_search_cb(self, model, column, key, it):
-        """Callback function for searching in VcView treeview"""
-        path = model.value_path(it, 0)
-
-        # if query text contains slash, search in full path
-        if key.find('/') >= 0:
-            lineText = path
-        else:
-            lineText = os.path.basename(path)
-
-        # Perform case-insensitive matching if query text is all lower-case
-        if key.islower():
-            lineText = lineText.lower()
-
-        if lineText.find(key) >= 0:
-            # line matches
-            return False
-        else:
-            return True
